==> emelfm2-0.1.2.tar.gz: emelfm2-0.1.2/src/dialogs/e2_date_filter_dialog.c <==
/*
 *
 * Copyright (C) 2003-2005 tooar <tooar@gmx.net>
 * Portions copyright (C) 1999 Michael Clark.
 *
 * This file is part of emelFM2.
 * emelFM2 is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 *
 * emelFM2 is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with emelFM2; see the file GPL. If not, contact the Free Software
 * Foundation, 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 */

#include "emelfm2.h"
//#define _XOPEN_SOURCE /* required for strptime in glibc-2 */
#include <time.h>
#include "e2_dialog.h"
#include "e2_filelist.h"

enum
{
	ACCESSED_SINCE,
	ACCESSED_BEFORE,
	MODIFIED_SINCE,
	MODIFIED_BEFORE,
	CHANGED_SINCE,
	CHANGED_BEFORE
};

static GtkWidget *dialog;
static GtkWidget *date_entry;
static GtkWidget *operation_combo;
//these need to be in same order as the date options in the config dialog
static gchar *date_format [4] =
{
	"%d/%m/%Y",	//default
	"%d/%m/%Y",	//standard
	"%m/%d/%Y",	//american
	"%x"	//localised
};
static gint date_index;	//the date format array index to use
static GtkWidget *menu_item;	//copy of pointer to clicked filters menu item
static gboolean itemstate;
//static gboolean blocked = FALSE;	//flag to prevent toggles when not wanted

/*static void _e2_date_filter_dialog_item_toggle_cb (void)
{
  gulong handler = g_signal_handler_find (menu_item, G_SIGNAL_MATCH_FUNC, 
	0, 0, NULL, e2_date_filter_dialog_create_cb, NULL);
  g_signal_handler_block (menu_item, handler);
  gtk_check_menu_item_set_active (GTK_CHECK_MENU_ITEM (menu_item), !itemstate);
  g_signal_handler_unblock (menu_item, handler);
}

static gboolean _e2_date_filter_dialog_destroy_cb (GtkWidget *widget)
{
	if (blocked)
	  blocked = FALSE;
  else
	  _e2_date_filter_dialog_item_toggle_cb ();
    return FALSE;
}

static gboolean _e2_date_filter_dialog_key_press_cb 
	(GtkWidget *widget, GdkEventKey *event)
{
  if (event->keyval == GDK_Escape)
    _e2_date_filter_dialog_destroy_cb (widget);
  return FALSE;
}
*/
static void _e2_date_filter_dialog_ok (FileView *view)
{
  struct tm tm_time;

  gchar *strf_withtime = g_strconcat (date_format [date_index] , " %T", NULL);
  gchar *date_string = g_strdup_printf ("%s 00:00:00",
	gtk_entry_get_text (GTK_ENTRY (date_entry)));
  strptime (date_string, strf_withtime, &tm_time);
  g_free (strf_withtime);
  g_free (date_string);
  view->date_filter.time = (time_t) mktime (&tm_time);

  gint index = gtk_combo_box_get_active (GTK_COMBO_BOX (operation_combo));
  //see below for the strings provided to the combo
  switch (index)
  {
	  case ACCESSED_SINCE:
		view->date_filter.op = GT;
		view->date_filter.time_type = ATIME;
		break;
	  case ACCESSED_BEFORE:
		view->date_filter.op = LT;
		view->date_filter.time_type = ATIME;
		break;
	  case MODIFIED_SINCE:
		view->date_filter.op = GT;
		view->date_filter.time_type = MTIME;
		break;
	  case MODIFIED_BEFORE:
		view->date_filter.op = LT;
		view->date_filter.time_type = MTIME;
		break;
	  case CHANGED_SINCE:
		view->date_filter.op = GT;
		view->date_filter.time_type = CTIME;
		break;
	  case CHANGED_BEFORE:
		view->date_filter.op = LT;
		view->date_filter.time_type = CTIME;
		break;
	  default:
		break;
  }

//  if (! itemstate)
	 //we're staying filtered, so re-toggle the menu item
//	_e2_date_filter_dialog_item_toggle_cb ();
  //and conform the cached copy
  view->date_filter.active = TRUE;

  //show the results
  e2_filelist_filter_toggle (view);
  if (index > -1)
	e2_filelist_refresh_list (view);
}

static void _e2_date_filter_dialog_response_cb (GtkWidget *button,
	gint response, FileView *view)
{
	switch (response)
	{
		case GTK_RESPONSE_OK:
			_e2_date_filter_dialog_ok (view);
		  	//prevent toggle when dialog is destroyed
//			blocked = TRUE;
		  break;
		case E2_RESPONSE_REMOVE:
		  //update the cached copy of the state
		  view->date_filter.active = FALSE;
		  //show the results
		  e2_filelist_filter_toggle (view);
		  e2_filelist_refresh_list (view);
		  //prevent toggle when dialog is destroyed
//		  blocked = TRUE;
		  break;
//		case GTK_RESPONSE_CANCEL:
		default:
		  //the menu item will be re-toggled when dialog is destroyed
		break;
	}
	gtk_widget_destroy (dialog);	
	gtk_main_quit ();
	return;
}
/**
 * @brief create file date filter dialog
 *
 * The state of @a item when it arrives here is opposite to that
 * shown in the menu, when clicked
 *
 * @param item the clicked checkbox widget in the fillters menu
 * @param view data structure for the view to which the file list belongs
 *
 * @return
 */
void e2_date_filter_dialog_create_cb (GtkWidget *item, FileView *view)
{
//  GtkWidget *dialog; use static instead
  GtkWidget *hbox;
  gchar date_string[16];
  struct tm *tm_ptr;
  gint index;
  
  //save local copies, for later use in other functions
  menu_item = item;
  itemstate = gtk_check_menu_item_get_active(GTK_CHECK_MENU_ITEM (item));

  dialog = e2_dialog_create (NULL, _("Display only the items:"),
		_("date filter"), _e2_date_filter_dialog_response_cb, view);
  hbox = add_hbox (GTK_DIALOG (dialog)->vbox, FALSE, E2_PADDING, TRUE, E2_PADDING);

  operation_combo = gtk_combo_box_new_text ();
  // don't change the order of these - index value is used below and in callback
  gtk_combo_box_append_text (GTK_COMBO_BOX(operation_combo), _("accessed since"));
  gtk_combo_box_append_text (GTK_COMBO_BOX(operation_combo), _("accessed before"));
  gtk_combo_box_append_text (GTK_COMBO_BOX(operation_combo), _("modified since"));
  gtk_combo_box_append_text (GTK_COMBO_BOX(operation_combo), _("modified before"));
  gtk_combo_box_append_text (GTK_COMBO_BOX(operation_combo), _("changed since"));
  gtk_combo_box_append_text (GTK_COMBO_BOX(operation_combo), _("changed before"));

  switch (view->date_filter.time_type)
  {
    case ATIME:
	  index = (view->date_filter.op == GT) ? ACCESSED_SINCE : ACCESSED_BEFORE; //0 = 1st combo string
      break;
    case MTIME:
      index = (view->date_filter.op == GT) ? MODIFIED_SINCE : MODIFIED_BEFORE;
      break;
    case CTIME:
      index = (view->date_filter.op == GT) ? CHANGED_SINCE : CHANGED_BEFORE;
      break;
    default:
      index = ACCESSED_SINCE;
      break;
  }
  gtk_combo_box_set_active (GTK_COMBO_BOX (operation_combo), index) ;

  gtk_box_pack_start (GTK_BOX (hbox), operation_combo, FALSE, FALSE, 0);
  gtk_widget_show (operation_combo);

  //get which date format to use
  date_index = e2_option_int_get ("date-string");
  if (date_index > 3)
	date_index = 0;	//out of range, use default format

  tm_ptr = localtime (&(view->date_filter.time));
  strftime (date_string, sizeof (date_string), date_format[date_index], tm_ptr);
  date_entry = add_entry (hbox, date_string, FALSE, 0);
  gtk_widget_set_size_request (date_entry, 120, 30);

  //we also need to intercept these
/*  g_signal_connect (G_OBJECT (dialog), "key_press_event",
	G_CALLBACK (_e2_date_filter_dialog_key_press_cb), NULL);
  g_signal_connect (G_OBJECT (dialog), "destroy",
	G_CALLBACK (_e2_date_filter_dialog_destroy_cb), NULL);
*/
  e2_dialog_setup (dialog, app.main_window);
//  gtk_widget_show (dialog);
//  gtk_widget_set_sensitive (app.main_window, FALSE);
  //now the buttons
  if (!itemstate)
	  //user has just toggled off
	  e2_dialog_add_defined_button (dialog, &E2_BUTTON_REMOVE);
  	//non-modal dialog
	e2_dialog_show (dialog, app.main_window,
		E2_DIALOG_FREE,  //no  _RUN (hence window stays sensitive)
		&E2_BUTTON_CANCEL, &E2_BUTTON_OK, NULL);
	gtk_main ();
}

==> emelfm2-0.1.4.tar.gz: emelfm2-0.1.4/src/dialogs/e2_date_filter_dialog.c <==
/* $Id: e2_date_filter_dialog.c 1219 2005-10-01 11:23:45Z tpgww $
 *
 * Copyright (C) 2003-2005 tooar <tooar@gmx.net>
 * Portions copyright (C) 1999 Michael Clark.
 *
 * This file is part of emelFM2.
 * emelFM2 is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 *
 * emelFM2 is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with emelFM2; see the file GPL. If not, contact the Free Software
 * Foundation, 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 */

#include "emelfm2.h"
//#define _XOPEN_SOURCE /* required for strptime in glibc-2 */
#include <time.h>
#include "e2_dialog.h"
#include "e2_filelist.h"

enum
{
	ACCESSED_SINCE,
	ACCESSED_BEFORE,
	MODIFIED_SINCE,
	MODIFIED_BEFORE,
	CHANGED_SINCE,
	CHANGED_BEFORE
};

static GtkWidget *dialog;
static GtkWidget *date_entry;
static GtkWidget *operation_combo;
//these need to be in same order as the date options in the config dialog
static gchar *date_format [4] =
{
	"%d/%m/%Y",	//default
	"%d/%m/%Y",	//standard
	"%m/%d/%Y",	//american
	"%x"	//localised
};
static gint date_index;	//the date format array index to use
static GtkWidget *menu_item;	//copy of pointer to clicked filters menu item
static gboolean itemstate;
//static gboolean blocked = FALSE;	//flag to prevent toggles when not wanted

/*static void _e2_date_filter_dialog_item_toggle_cb (void)
{
  gulong handler = g_signal_handler_find (menu_item, G_SIGNAL_MATCH_FUNC, 
	0, 0, NULL, e2_date_filter_dialog_create_cb, NULL);
  g_signal_handler_block (menu_item, handler);
  gtk_check_menu_item_set_active (GTK_CHECK_MENU_ITEM (menu_item), !itemstate);
  g_signal_handler_unblock (menu_item, handler);
}

static gboolean _e2_date_filter_dialog_destroy_cb (GtkWidget *widget)
{
	if (blocked)
	  blocked = FALSE;
  else
	  _e2_date_filter_dialog_item_toggle_cb ();
    return FALSE;
}

static gboolean _e2_date_filter_dialog_key_press_cb 
	(GtkWidget *widget, GdkEventKey *event)
{
  if (event->keyval == GDK_Escape)
    _e2_date_filter_dialog_destroy_cb (widget);
  return FALSE;
}
*/
static void _e2_date_filter_dialog_ok (FileView *view)
{
  struct tm tm_time;

  gchar *strf_withtime = g_strconcat (date_format [date_index] , " %T", NULL);
  gchar *date_string = g_strdup_printf ("%s 00:00:00",
	gtk_entry_get_text (GTK_ENTRY (date_entry)));
  strptime (date_string, strf_withtime, &tm_time);
  g_free (strf_withtime);
  g_free (date_string);
  view->date_filter.time = (time_t) mktime (&tm_time);

  gint index = gtk_combo_box_get_active (GTK_COMBO_BOX (operation_combo));
  //see below for the strings provided to the combo
  switch (index)
  {
	  case ACCESSED_SINCE:
		view->date_filter.op = GT;
		view->date_filter.time_type = ATIME;
		break;
	  case ACCESSED_BEFORE:
		view->date_filter.op = LT;
		view->date_filter.time_type = ATIME;
		break;
	  case MODIFIED_SINCE:
		view->date_filter.op = GT;
		view->date_filter.time_type = MTIME;
		break;
	  case MODIFIED_BEFORE:
		view->date_filter.op = LT;
		view->date_filter.time_type = MTIME;
		break;
	  case CHANGED_SINCE:
		view->date_filter.op = GT;
		view->date_filter.time_type = CTIME;
		break;
	  case CHANGED_BEFORE:
		view->date_filter.op = LT;
		view->date_filter.time_type = CTIME;
		break;
	  default:
		break;
  }

//  if (! itemstate)
	 //we're staying filtered, so re-toggle the menu item
//	_e2_date_filter_dialog_item_toggle_cb ();
  //and conform the cached copy
  view->date_filter.active = TRUE;

  //show the results
  e2_filelist_filter_toggle (view);
  if (index > -1)
	e2_fileview_refresh_list (view);
}

static void _e2_date_filter_dialog_response_cb (GtkWidget *button,
	gint response, FileView *view)
{
	switch (response)
	{
		case GTK_RESPONSE_OK:
			_e2_date_filter_dialog_ok (view);
		  	//prevent toggle when dialog is destroyed
//			blocked = TRUE;
		  break;
		case E2_RESPONSE_REMOVE:
		  //update the cached copy of the state
		  view->date_filter.active = FALSE;
		  //show the results
		  e2_filelist_filter_toggle (view);
		  e2_fileview_refresh_list (view);
		  //prevent toggle when dialog is destroyed
//		  blocked = TRUE;
		  break;
//		case GTK_RESPONSE_CANCEL:
		default:
		  //the menu item will be re-toggled when dialog is destroyed
		break;
	}
	gtk_widget_destroy (dialog);	
	gtk_main_quit ();
	return;
}
/**
 * @brief create file date filter dialog
 *
 * The state of @a item when it arrives here is opposite to that
 * shown in the menu, when clicked
 *
 * @param item the clicked checkbox widget in the fillters menu
 * @param view data structure for the view to which the file list belongs
 *
 * @return
 */
void e2_date_filter_dialog_create_cb (GtkWidget *item, FileView *view)
{
//  GtkWidget *dialog; use static instead
  GtkWidget *hbox;
  gchar date_string[16];
  struct tm *tm_ptr;
  gint index;
  
  //save local copies, for later use in other functions
  menu_item = item;
  itemstate = gtk_check_menu_item_get_active(GTK_CHECK_MENU_ITEM (item));

  dialog = e2_dialog_create (NULL, _("Display only the items:"),
		_("date filter"), _e2_date_filter_dialog_response_cb, view);
  hbox = add_hbox (GTK_DIALOG (dialog)->vbox, FALSE, E2_PADDING, TRUE, E2_PADDING);

  operation_combo = gtk_combo_box_new_text ();
  // don't change the order of these - index value is used below and in callback
  gtk_combo_box_append_text (GTK_COMBO_BOX(operation_combo), _("accessed since"));
  gtk_combo_box_append_text (GTK_COMBO_BOX(operation_combo), _("accessed before"));
  gtk_combo_box_append_text (GTK_COMBO_BOX(operation_combo), _("modified since"));
  gtk_combo_box_append_text (GTK_COMBO_BOX(operation_combo), _("modified before"));
  gtk_combo_box_append_text (GTK_COMBO_BOX(operation_combo), _("changed since"));
  gtk_combo_box_append_text (GTK_COMBO_BOX(operation_combo), _("changed before"));

  switch (view->date_filter.time_type)
  {
    case ATIME:
	  index = (view->date_filter.op == GT) ? ACCESSED_SINCE : ACCESSED_BEFORE; //0 = 1st combo string
      break;
    case MTIME:
      index = (view->date_filter.op == GT) ? MODIFIED_SINCE : MODIFIED_BEFORE;
      break;
    case CTIME:
      index = (view->date_filter.op == GT) ? CHANGED_SINCE : CHANGED_BEFORE;
      break;
    default:
      index = ACCESSED_SINCE;
      break;
  }
  gtk_combo_box_set_active (GTK_COMBO_BOX (operation_combo), index) ;

  gtk_box_pack_start (GTK_BOX (hbox), operation_combo, FALSE, FALSE, 0);
  gtk_widget_show (operation_combo);

  //get which date format to use
  date_index = e2_option_int_get ("date-string");
  if (date_index > 3)
	date_index = 0;	//out of range, use default format

  tm_ptr = localtime (&(view->date_filter.time));
  strftime (date_string, sizeof (date_string), date_format[date_index], tm_ptr);
  date_entry = add_entry (hbox, date_string, FALSE, 0);
  gtk_widget_set_size_request (date_entry, 120, 30);

  //we also need to intercept these
/*  g_signal_connect (G_OBJECT (dialog), "key_press_event",
	G_CALLBACK (_e2_date_filter_dialog_key_press_cb), NULL);
  g_signal_connect (G_OBJECT (dialog), "destroy",
	G_CALLBACK (_e2_date_filter_dialog_destroy_cb), NULL);
*/
  e2_dialog_setup (dialog, app.main_window);
//  gtk_widget_show (dialog);
//  gtk_widget_set_sensitive (app.main_window, FALSE);
  //now the buttons
  if (!itemstate)
	  //user has just toggled off
	  e2_dialog_add_defined_button (dialog, &E2_BUTTON_REMOVE);
  	//non-modal dialog
	e2_dialog_show (dialog, app.main_window,
		E2_DIALOG_FREE,  //no  _RUN (hence window stays sensitive)
		&E2_BUTTON_CANCEL, &E2_BUTTON_OK, NULL);
	gtk_main ();
}
